"use strict";
var common = require("./carousel-view-common");
var observable = require("data/observable");
var color_1 = require("color");
var CarouselView = (function (_super) {
    __extends(CarouselView, _super);
    function CarouselView() {
        var _this = _super.call(this) || this;
        _this._layoutCount = 0;
        // As custom properties returns default value only here
        // Adding a container so we can build UIPageViewController onLoaded()
        _this._ios = new UIView();
        _this._ios.clipsToBounds = true; // to avoid pageController view go outside the bounds of the container.
        return _this;
    }
    Object.defineProperty(CarouselView.prototype, "ios", {
        get: function () {
            return this._ios;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CarouselView.prototype, "_nativeView", {
        // Thanks to NathanWalker for the _nativeView tip
        get: function () {
            return this._ios;
        },
        enumerable: true,
        configurable: true
    });
    // Thanks to NathanWalker for the onLoaded tip
    CarouselView.prototype.onLoaded = function () {
        this.configurePageController();
        this._pageControl = new UIPageControl();
        this._pageControl.pageIndicatorTintColor = new color_1.Color(this.indicatorsTintColor).ios;
        this._pageControl.currentPageIndicatorTintColor = new color_1.Color(this.indicatorsCurrentPageColor).ios;
        this._pageControl.translatesAutoresizingMaskIntoConstraints = false;
        this._pageControl.enabled = false;
        this._ios.addSubview(this._pageControl);
        var viewsDictionary = NSDictionary.dictionaryWithObjectForKey(this._pageControl, "pageControl");
        switch (this.orientation) {
            case "horizontal":
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("H:|-[pageControl]-|", NSLayoutFormatOptions.NSLayoutFormatAlignAllCenterX, null, viewsDictionary));
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("V:[pageControl]|", 0, null, viewsDictionary));
                break;
            case "vertical":
                this._pageControl.transform = CGAffineTransformMakeRotation(3.14159265 / 2);
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("[pageControl(==36)]", 0, null, viewsDictionary));
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("H:[pageControl]|", 0, null, viewsDictionary));
                this._ios.addConstraints(NSLayoutConstraint.constraintsWithVisualFormatOptionsMetricsViews("V:|-[pageControl]-|", NSLayoutFormatOptions.NSLayoutFormatAlignAllCenterY, null, viewsDictionary));
                break;
        }
        this._pageControl.hidden = !this.showIndicators;
        var eventData = {
            eventName: "positionSelected",
            object: this
        };
        this.notify(eventData);
        this.on(observable.Observable.propertyChangeEvent, function (propertyChangeData) {
            switch (propertyChangeData.propertyName) {
                case "itemsSource":
                    if (this._pageController != null) {
                        this.configurePageController();
                        this.configurePageControl();
                        var eventData = {
                            eventName: "positionSelected",
                            object: this
                        };
                        this.notify(eventData);
                    }
                    break;
                case "showIndicators":
                    this._pageControl.hidden = !propertyChangeData.value;
                    break;
            }
        }, this);
    };
    CarouselView.prototype.configurePageController = function () {
        if (this.itemsSource != null) {
            if (this.position > this.itemsSource.length - 1)
                this.position = this.itemsSource.length - 1;
        }
        else {
            this.position = 0;
        }
        if (this.position == -1)
            this.position = 0;
        var orientation;
        switch (this.orientation) {
            case "horizontal":
                orientation = UIPageViewControllerNavigationOrientation.UIPageViewControllerNavigationOrientationHorizontal;
                break;
            case "vertical":
                orientation = UIPageViewControllerNavigationOrientation.UIPageViewControllerNavigationOrientationVertical;
                break;
            default:
                throw new Error("CarouselView " + this.orientation + " orientation is not supported.");
        }
        var objects = [UIPageViewControllerSpineLocation.UIPageViewControllerSpineLocationNone, this.interPageSpacing];
        var keys = [UIPageViewControllerOptionSpineLocationKey, UIPageViewControllerOptionInterPageSpacingKey];
        this._pageController = UIPageViewController.alloc().initWithTransitionStyleNavigationOrientationOptions(UIPageViewControllerTransitionStyle.UIPageViewControllerTransitionStyleScroll, orientation, NSDictionary.dictionaryWithObjectsForKeys(objects, keys));
        var that = new WeakRef(this);
        this._pageController.dataSource = DataSourceClass.initWithOwner(that);
        this._pageController.delegate = DelegateClass.initWithOwner(that);
        if (this.itemsSource != null) {
            if (this.itemsSource.length > 0) {
                var firstViewController = this.createViewController(this.position);
                var direction = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
                this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, false, function (arg1) { });
            }
        }
        // Property for the user set the margin color (background color)
        this._pageController.view.backgroundColor = new color_1.Color(this.interPageSpacingColor).ios;
        if (this._ios.subviews.count > 0)
            this._ios.subviews.objectAtIndex(0).removeFromSuperview();
        this._ios.insertSubviewAtIndex(this._pageController.view, 0);
    };
    CarouselView.prototype.configurePageControl = function () {
        if (this._pageControl != null) {
            var count = this.itemsSource != null ? this.itemsSource.length : 0;
            this._pageControl.numberOfPages = count;
            this._pageControl.currentPage = this.position;
            switch (this.indicatorsShape) {
                case "Circle":
                    // do nothing, default
                    break;
                case "Square":
                    for (var i = 0; i < this._pageControl.subviews.count; i++) {
                        var view = this._pageControl.subviews[i];
                        view.layer.cornerRadius = 0;
                        if (view.frame.size.width == 7) {
                            var frame = CGRectMake(view.frame.origin.x, view.frame.origin.y, view.frame.size.width - 1, view.frame.size.height - 1);
                            view.frame = frame;
                        }
                    }
                    break;
                default:
                    throw new Error("CarouselView " + this.indicatorsShape + " indicatorsShape is not supported.");
            }
        }
    };
    CarouselView.prototype.onLayout = function (left, top, right, bottom) {
        if (this._layoutCount == 1) {
            this.configurePageControl();
        }
        this._layoutCount++;
    };
    CarouselView.prototype.insertPage = function (position, bindingContext) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var firstViewController, direction;
            return __generator(this, function (_a) {
                if (this._pageController != null && this.itemsSource != null) {
                    if (position > this.itemsSource.length)
                        throw new Error("Index out of bounds (position > itemsSource length).");
                    if (position < 0)
                        throw new Error("Index out of bounds (position < 0).");
                    if (position == this.itemsSource.length)
                        this.itemsSource.push(bindingContext);
                    else
                        this.itemsSource.splice(position, 0, bindingContext);
                    firstViewController = void 0;
                    if (this._pageController.viewControllers.count > 0)
                        firstViewController = this._pageController.viewControllers[0];
                    else
                        firstViewController = this.createViewController(0);
                    direction = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
                    // Using a standard JS Array here (just type-cast to any to suffice TypeScript)
                    // {N} will auto-marshall this into a NSArray when making the call since the metadata knows its
                    // supposed to be an NSArray :)
                    // Thanks to NathanWalker for the auto-marshall tip for NSArray
                    this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, false, function (arg1) { return __awaiter(_this, void 0, void 0, function () {
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    this.configurePageControl();
                                    return [4 /*yield*/, this.delay(100)];
                                case 1:
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    }); });
                }
                return [2 /*return*/];
            });
        });
    };
    CarouselView.prototype.removePage = function (position) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var newPos, forward, reverse, direction, firstViewController, firstViewController, direction;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._pageController != null && this.itemsSource != null))
                            return [3 /*break*/, 3];
                        if (!(this.itemsSource.length > 0))
                            return [3 /*break*/, 3];
                        if (position > this.itemsSource.length - 1)
                            throw new Error("Index out of bounds (position > itemsSource length - 1).");
                        if (position < 0)
                            throw new Error("Index out of bounds (position < 0).");
                        this.itemsSource.splice(position, 1);
                        if (!(position == this.position))
                            return [3 /*break*/, 2];
                        newPos = position - 1;
                        if (newPos == -1)
                            newPos = 0;
                        return [4 /*yield*/, this.delay(100)];
                    case 1:
                        _a.sent();
                        forward = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
                        reverse = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionReverse;
                        direction = position == 0 ? forward : reverse;
                        firstViewController = this.createViewController(newPos);
                        this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, this.animateTransition, function (arg1) {
                            _this.position = newPos;
                            _this.configurePageControl();
                            var eventData = {
                                eventName: "positionSelected",
                                object: _this
                            };
                            _this.notify(eventData);
                        });
                        return [3 /*break*/, 3];
                    case 2:
                        firstViewController = this._pageController.viewControllers[0];
                        direction = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
                        this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, false, function (arg1) {
                            _this.configurePageControl();
                            var eventData = {
                                eventName: "positionSelected",
                                object: _this
                            };
                            _this.notify(eventData);
                        });
                        _a.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    CarouselView.prototype.setCurrentPage = function (position) {
        var _this = this;
        if (this._pageController != null && this.itemsSource != null) {
            if (this.itemsSource.length > 0) {
                if (position > this.itemsSource.length - 1)
                    throw new Error("Index out of bounds (position > itemsSource length - 1).");
                if (position < 0)
                    throw new Error("Index out of bounds (position < 0).");
                var forward = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionForward;
                var reverse = UIPageViewControllerNavigationDirection.UIPageViewControllerNavigationDirectionReverse;
                var direction = position > this.position ? forward : reverse;
                this.position = position;
                var firstViewController = this.createViewController(position);
                this._pageController.setViewControllersDirectionAnimatedCompletion([firstViewController], direction, this.animateTransition, function (arg1) {
                    _this.configurePageControl();
                    var eventData = {
                        eventName: "positionSelected",
                        object: _this
                    };
                    _this.notify(eventData);
                });
            }
        }
    };
    CarouselView.prototype.createViewController = function (position) {
        var item;
        if (this.itemsSource != null) {
            if (this.itemsSource.length > 0) {
                item = this.itemsSource.getItem(position);
            }
        }
        var view = this.templateSelector.OnSelectTemplate(position, item);
        var obj = view;
        this._addView(obj);
        var viewController = new ViewContainer();
        viewController.tag = position;
        viewController.view = obj._view;
        viewController.owner = view;
        obj.onLoaded();
        return viewController;
    };
    CarouselView.prototype.delay = function (ms) {
        return new Promise(function (resolve) { return setTimeout(resolve, ms); });
    };
    return CarouselView;
}(common.CarouselView));
exports.CarouselView = CarouselView;
var DataSourceClass = (function (_super) {
    __extends(DataSourceClass, _super);
    function DataSourceClass() {
        return _super.apply(this, arguments) || this;
    }
    Object.defineProperty(DataSourceClass.prototype, "owner", {
        get: function () {
            return this._owner.get();
        },
        enumerable: true,
        configurable: true
    });
    DataSourceClass.initWithOwner = function (owner) {
        var datasource = new DataSourceClass();
        datasource._owner = owner;
        return datasource;
    };
    DataSourceClass.prototype.pageViewControllerViewControllerBeforeViewController = function (pageViewController, viewController) {
        var controller = viewController;
        if (controller != null) {
            var position = Number(controller.tag);
            // Determine if we are on the first page
            if (position == 0) {
                // We are on the first page, so there is no need for a controller before that
                return null;
            }
            else {
                var previousPageIndex = position - 1;
                return this.owner.createViewController(previousPageIndex);
            }
        }
        else {
            return null;
        }
    };
    DataSourceClass.prototype.pageViewControllerViewControllerAfterViewController = function (pageViewController, viewController) {
        var controller = viewController;
        if (controller != null) {
            var position = Number(controller.tag);
            // Determine if we are on the last page
            var count = this.presentationCountForPageViewController(pageViewController);
            if (position == count - 1) {
                // We are on the last page, so there is no need for a controller after that
                return null;
            }
            else {
                var nextPageIndex = position + 1;
                return this.owner.createViewController(nextPageIndex);
            }
        }
        else {
            return null;
        }
    };
    DataSourceClass.prototype.presentationCountForPageViewController = function (pageViewController) {
        // FIX: populate the carousel with data after being loaded in UI
        if (this.owner.itemsSource == null)
            return 0;
        return this.owner.itemsSource.length;
    };
    return DataSourceClass;
}(NSObject));
DataSourceClass.ObjCProtocols = [UIPageViewControllerDataSource];
var DelegateClass = (function (_super) {
    __extends(DelegateClass, _super);
    function DelegateClass() {
        return _super.apply(this, arguments) || this;
    }
    Object.defineProperty(DelegateClass.prototype, "owner", {
        get: function () {
            return this._owner.get();
        },
        enumerable: true,
        configurable: true
    });
    DelegateClass.initWithOwner = function (owner) {
        var delegate = new DelegateClass();
        delegate._owner = owner;
        return delegate;
    };
    DelegateClass.prototype.pageViewControllerDidFinishAnimatingPreviousViewControllersTransitionCompleted = function (pageViewController, finished, previousViewControllers, completed) {
        if (finished) {
            var controller = pageViewController.viewControllers[0];
            var position = controller.tag;
            this.owner.position = position;
            this.owner.configurePageControl();
            var eventData = {
                eventName: "positionSelected",
                object: this.owner
            };
            this.owner.notify(eventData);
        }
    };
    return DelegateClass;
}(NSObject));
DelegateClass.ObjCProtocols = [UIPageViewControllerDelegate];
var ViewContainer = (function (_super) {
    __extends(ViewContainer, _super);
    function ViewContainer() {
        return _super.apply(this, arguments) || this;
    }
    // Thanks to tzraikov
    ViewContainer.prototype.viewDidLoad = function () {
        _super.prototype.viewDidLoad.call(this);
        if (this.owner) {
            this.owner.onLoaded();
        }
    };
    // Thanks to tzraikov
    ViewContainer.prototype.viewDidLayoutSubviews = function () {
        if (this.owner) {
            var width = this.view.frame.size.width;
            var height = this.view.frame.size.height;
            this.owner.measure(width, height);
            this.owner.layout(0, 0, width, height);
            this.owner._updateLayout();
        }
    };
    return ViewContainer;
}(UIViewController));
exports.ViewContainer = ViewContainer;
//# sourceMappingURL=carousel-view.ios.js.map